@@ -24,34 +24,27 @@ Authors:
 # Imports
 #-----------------------------------------------------------------------------
 
-import __builtin__
-import codeop
 import re
 
-from IPython.core.alias import AliasManager
 from IPython.core.autocall import IPyAutocall
 from IPython.config.configurable import Configurable
+from IPython.core.inputsplitter import (
+    ESC_MAGIC,
+    ESC_QUOTE,
+    ESC_QUOTE2,
+    ESC_PAREN,
+)
 from IPython.core.macro import Macro
-from IPython.core.splitinput import split_user_input, LineInfo
-from IPython.core import page
+from IPython.core.splitinput import LineInfo
 
-from IPython.utils.traitlets import List, Integer, Any, Unicode, CBool, Bool, Instance
-from IPython.utils.autoattr import auto_attr
+from IPython.utils.traitlets import (
+    List, Integer, Unicode, CBool, Bool, Instance, CRegExp
+)
 
 #-----------------------------------------------------------------------------
 # Global utilities, errors and constants
 #-----------------------------------------------------------------------------
 
-# Warning, these cannot be changed unless various regular expressions
-# are updated in a number of places.  Not great, but at least we told you.
-ESC_SHELL  = '!'
-ESC_SH_CAP = '!!'
-ESC_HELP   = '?'
-ESC_MAGIC  = '%'
-ESC_QUOTE  = ','
-ESC_QUOTE2 = ';'
-ESC_PAREN  = '/'
-
 
 class PrefilterError(Exception):
     pass
@@ -113,8 +106,7 @@ class PrefilterManager(Configurable):
     The transformers are instances of :class:`PrefilterTransformer` and have
     a single method :meth:`transform` that takes a line and returns a
     transformed line.  The transformation can be accomplished using any
-    tool, but our current ones use regular expressions for speed.  We also
-    ship :mod:`pyparsing` in :mod:`IPython.external` for use in transformers.
+    tool, but our current ones use regular expressions for speed.
 
     After all the transformers have been run, the line is fed to the checkers,
     which are instances of :class:`PrefilterChecker`.  The line is passed to
@@ -138,8 +130,8 @@ class PrefilterManager(Configurable):
     multi_line_specials = CBool(True, config=True)
     shell = Instance('IPython.core.interactiveshell.InteractiveShellABC')
 
-    def __init__(self, shell=None, config=None):
-        super(PrefilterManager, self).__init__(shell=shell, config=config)
+    def __init__(self, shell=None, **kwargs):
+        super(PrefilterManager, self).__init__(shell=shell, **kwargs)
         self.shell = shell
         self.init_transformers()
         self.init_handlers()
@@ -154,7 +146,7 @@ class PrefilterManager(Configurable):
         self._transformers = []
         for transformer_cls in _default_transformers:
             transformer_cls(
-                shell=self.shell, prefilter_manager=self, config=self.config
+                shell=self.shell, prefilter_manager=self, parent=self
             )
 
     def sort_transformers(self):
@@ -190,7 +182,7 @@ class PrefilterManager(Configurable):
         self._checkers = []
         for checker in _default_checkers:
             checker(
-                shell=self.shell, prefilter_manager=self, config=self.config
+                shell=self.shell, prefilter_manager=self, parent=self
             )
 
     def sort_checkers(self):
@@ -227,7 +219,7 @@ class PrefilterManager(Configurable):
         self._esc_handlers = {}
         for handler in _default_handlers:
             handler(
-                shell=self.shell, prefilter_manager=self, config=self.config
+                shell=self.shell, prefilter_manager=self, parent=self
             )
 
     @property
@@ -322,9 +314,6 @@ class PrefilterManager(Configurable):
 
         normal_handler = self.get_handler_by_name('normal')
         if not stripped:
-            if not continue_prompt:
-                self.shell.displayhook.prompt_count -= 1
-
             return normal_handler.handle(line_info)
 
         # special handlers are only allowed for single line statements
@@ -375,9 +364,9 @@ class PrefilterTransformer(Configurable):
     prefilter_manager = Instance('IPython.core.prefilter.PrefilterManager')
     enabled = Bool(True, config=True)
 
-    def __init__(self, shell=None, prefilter_manager=None, config=None):
+    def __init__(self, shell=None, prefilter_manager=None, **kwargs):
         super(PrefilterTransformer, self).__init__(
-            shell=shell, prefilter_manager=prefilter_manager, config=config
+            shell=shell, prefilter_manager=prefilter_manager, **kwargs
         )
         self.prefilter_manager.register_transformer(self)
 
@@ -390,85 +379,6 @@ class PrefilterTransformer(Configurable):
             self.__class__.__name__, self.priority, self.enabled)
 
 
-_assign_system_re = re.compile(r'(?P<lhs>(\s*)([\w\.]+)((\s*,\s*[\w\.]+)*))'
-                               r'\s*=\s*!(?P<cmd>.*)')
-
-
-class AssignSystemTransformer(PrefilterTransformer):
-    """Handle the `files = !ls` syntax."""
-
-    priority = Integer(100, config=True)
-
-    def transform(self, line, continue_prompt):
-        m = _assign_system_re.match(line)
-        if m is not None:
-            cmd = m.group('cmd')
-            lhs = m.group('lhs')
-            expr = "sc =%s" % cmd
-            new_line = '%s = get_ipython().magic(%r)' % (lhs, expr)
-            return new_line
-        return line
-
-
-_assign_magic_re = re.compile(r'(?P<lhs>(\s*)([\w\.]+)((\s*,\s*[\w\.]+)*))'
-                               r'\s*=\s*%(?P<cmd>.*)')
-
-class AssignMagicTransformer(PrefilterTransformer):
-    """Handle the `a = %who` syntax."""
-
-    priority = Integer(200, config=True)
-
-    def transform(self, line, continue_prompt):
-        m = _assign_magic_re.match(line)
-        if m is not None:
-            cmd = m.group('cmd')
-            lhs = m.group('lhs')
-            new_line = '%s = get_ipython().magic(%r)' % (lhs, cmd)
-            return new_line
-        return line
-
-
-_classic_prompt_re = re.compile(r'(^[ \t]*>>> |^[ \t]*\.\.\. )')
-
-class PyPromptTransformer(PrefilterTransformer):
-    """Handle inputs that start with '>>> ' syntax."""
-
-    priority = Integer(50, config=True)
-
-    def transform(self, line, continue_prompt):
-
-        if not line or line.isspace() or line.strip() == '...':
-            # This allows us to recognize multiple input prompts separated by
-            # blank lines and pasted in a single chunk, very common when
-            # pasting doctests or long tutorial passages.
-            return ''
-        m = _classic_prompt_re.match(line)
-        if m:
-            return line[len(m.group(0)):]
-        else:
-            return line
-
-
-_ipy_prompt_re = re.compile(r'(^[ \t]*In \[\d+\]: |^[ \t]*\ \ \ \.\.\.+: )')
-
-class IPyPromptTransformer(PrefilterTransformer):
-    """Handle inputs that start classic IPython prompt syntax."""
-
-    priority = Integer(50, config=True)
-
-    def transform(self, line, continue_prompt):
-
-        if not line or line.isspace() or line.strip() == '...':
-            # This allows us to recognize multiple input prompts separated by
-            # blank lines and pasted in a single chunk, very common when
-            # pasting doctests or long tutorial passages.
-            return ''
-        m = _ipy_prompt_re.match(line)
-        if m:
-            return line[len(m.group(0)):]
-        else:
-            return line
-
 #-----------------------------------------------------------------------------
 # Prefilter checkers
 #-----------------------------------------------------------------------------
@@ -482,9 +392,9 @@ class PrefilterChecker(Configurable):
     prefilter_manager = Instance('IPython.core.prefilter.PrefilterManager')
     enabled = Bool(True, config=True)
 
-    def __init__(self, shell=None, prefilter_manager=None, config=None):
+    def __init__(self, shell=None, prefilter_manager=None, **kwargs):
         super(PrefilterChecker, self).__init__(
-            shell=shell, prefilter_manager=prefilter_manager, config=config
+            shell=shell, prefilter_manager=prefilter_manager, **kwargs
         )
         self.prefilter_manager.register_checker(self)
 
@@ -510,15 +420,6 @@ class EmacsChecker(PrefilterChecker):
             return None
 
 
-class ShellEscapeChecker(PrefilterChecker):
-
-    priority = Integer(200, config=True)
-
-    def check(self, line_info):
-        if line_info.line.lstrip().startswith(ESC_SHELL):
-            return self.prefilter_manager.get_handler_by_name('shell')
-
-
 class MacroChecker(PrefilterChecker):
 
     priority = Integer(250, config=True)
@@ -545,43 +446,6 @@ class IPyAutocallChecker(PrefilterChecker):
             return None
 
 
-class MultiLineMagicChecker(PrefilterChecker):
-
-    priority = Integer(400, config=True)
-
-    def check(self, line_info):
-        "Allow ! and !! in multi-line statements if multi_line_specials is on"
-        # Note that this one of the only places we check the first character of
-        # ifun and *not* the pre_char.  Also note that the below test matches
-        # both ! and !!.
-        if line_info.continue_prompt \
-            and self.prefilter_manager.multi_line_specials:
-                if line_info.esc == ESC_MAGIC:
-                    return self.prefilter_manager.get_handler_by_name('magic')
-        else:
-            return None
-
-
-class EscCharsChecker(PrefilterChecker):
-
-    priority = Integer(500, config=True)
-
-    def check(self, line_info):
-        """Check for escape character and return either a handler to handle it,
-        or None if there is no escape char."""
-        if line_info.line[-1] == ESC_HELP \
-               and line_info.esc != ESC_SHELL \
-               and line_info.esc != ESC_SH_CAP:
-            # the ? can be at the end, but *not* for either kind of shell escape,
-            # because a ? can be a vaild final char in a shell cmd
-            return self.prefilter_manager.get_handler_by_name('help')
-        else:
-            if line_info.pre:
-                return None
-            # This returns None like it should if no handler exists
-            return self.prefilter_manager.get_handler_by_esc(line_info.esc)
-
-
 class AssignmentChecker(PrefilterChecker):
 
     priority = Integer(600, config=True)
@@ -610,7 +474,7 @@ class AutoMagicChecker(PrefilterChecker):
         check_esc_chars. This just checks for automagic.  Also, before
         triggering the magic handler, make sure that there is nothing in the
         user namespace which could shadow it."""
-        if not self.shell.automagic or not hasattr(self.shell,'magic_'+line_info.ifun):
+        if not self.shell.automagic or not self.shell.find_magic(line_info.ifun):
             return None
 
         # We have a likely magic method.  Make sure we should actually call it.
@@ -659,6 +523,11 @@ class AutocallChecker(PrefilterChecker):
 
     priority = Integer(1000, config=True)
 
+    function_name_regexp = CRegExp(re_fun_name, config=True,
+        help="RegExp to identify potential function names.")
+    exclude_regexp = CRegExp(re_exclude_auto, config=True,
+        help="RegExp to exclude strings with this start from autocalling.")
+
     def check(self, line_info):
         "Check if the initial word/function is callable and autocall is on."
         if not self.shell.autocall:
@@ -669,8 +538,8 @@ class AutocallChecker(PrefilterChecker):
             return None
 
         if callable(oinfo['obj']) \
-               and (not re_exclude_auto.match(line_info.the_rest)) \
-               and re_fun_name.match(line_info.ifun):
+               and (not self.exclude_regexp.match(line_info.the_rest)) \
+               and self.function_name_regexp.match(line_info.ifun):
             return self.prefilter_manager.get_handler_by_name('auto')
         else:
             return None
@@ -688,9 +557,9 @@ class PrefilterHandler(Configurable):
     shell = Instance('IPython.core.interactiveshell.InteractiveShellABC')
     prefilter_manager = Instance('IPython.core.prefilter.PrefilterManager')
 
-    def __init__(self, shell=None, prefilter_manager=None, config=None):
+    def __init__(self, shell=None, prefilter_manager=None, **kwargs):
         super(PrefilterHandler, self).__init__(
-            shell=shell, prefilter_manager=prefilter_manager, config=config
+            shell=shell, prefilter_manager=prefilter_manager, **kwargs
         )
         self.prefilter_manager.register_handler(
             self.handler_name,
@@ -736,33 +605,6 @@ class AliasHandler(PrefilterHandler):
         return line_out
 
 
-class ShellEscapeHandler(PrefilterHandler):
-
-    handler_name = Unicode('shell')
-    esc_strings = List([ESC_SHELL, ESC_SH_CAP])
-
-    def handle(self, line_info):
-        """Execute the line in a shell, empty return value"""
-        magic_handler = self.prefilter_manager.get_handler_by_name('magic')
-
-        line = line_info.line
-        if line.lstrip().startswith(ESC_SH_CAP):
-            # rewrite LineInfo's line, ifun and the_rest to properly hold the
-            # call to %sx and the actual command to be executed, so
-            # handle_magic can work correctly.  Note that this works even if
-            # the line is indented, so it handles multi_line_specials
-            # properly.
-            new_rest = line.lstrip()[2:]
-            line_info.line = '%ssx %s' % (ESC_MAGIC, new_rest)
-            line_info.ifun = 'sx'
-            line_info.the_rest = new_rest
-            return magic_handler.handle(line_info)
-        else:
-            cmd = line.lstrip().lstrip(ESC_SHELL)
-            line_out = '%sget_ipython().system(%r)' % (line_info.pre_whitespace, cmd)
-        return line_out
-
-
 class MacroHandler(PrefilterHandler):
     handler_name = Unicode("macro")
 
@@ -800,7 +642,7 @@ class AutoHandler(PrefilterHandler):
         pre     = line_info.pre
         esc     = line_info.esc
         continue_prompt = line_info.continue_prompt
-        obj = line_info.ofind(self)['obj']
+        obj = line_info.ofind(self.shell)['obj']
         #print 'pre <%s> ifun <%s> rest <%s>' % (pre,ifun,the_rest)  # dbg
 
         # This should only be active for single-line input!
@@ -859,44 +701,6 @@ class AutoHandler(PrefilterHandler):
         return newcmd
 
 
-class HelpHandler(PrefilterHandler):
-
-    handler_name = Unicode('help')
-    esc_strings = List([ESC_HELP])
-
-    def handle(self, line_info):
-        """Try to get some help for the object.
-
-        obj? or ?obj   -> basic information.
-        obj?? or ??obj -> more details.
-        """
-        normal_handler = self.prefilter_manager.get_handler_by_name('normal')
-        line = line_info.line
-        # We need to make sure that we don't process lines which would be
-        # otherwise valid python, such as "x=1 # what?"
-        try:
-            codeop.compile_command(line)
-        except SyntaxError:
-            # We should only handle as help stuff which is NOT valid syntax
-            if line[0]==ESC_HELP:
-                line = line[1:]
-            elif line[-1]==ESC_HELP:
-                line = line[:-1]
-            if line:
-                #print 'line:<%r>' % line  # dbg
-                self.shell.magic_pinfo(line_info.ifun)
-            else:
-                self.shell.show_usage()
-            return '' # Empty string is needed here!
-        except:
-            raise
-            # Pass any other exceptions through to the normal handler
-            return normal_handler.handle(line_info)
-        else:
-            # If the code compiles ok, we should handle it normally
-            return normal_handler.handle(line_info)
-
-
 class EmacsHandler(PrefilterHandler):
 
     handler_name = Unicode('emacs')
@@ -918,19 +722,12 @@ class EmacsHandler(PrefilterHandler):
 
 
 _default_transformers = [
-    AssignSystemTransformer,
-    AssignMagicTransformer,
-    PyPromptTransformer,
-    IPyPromptTransformer,
 ]
 
 _default_checkers = [
     EmacsChecker,
-    ShellEscapeChecker,
     MacroChecker,
     IPyAutocallChecker,
-    MultiLineMagicChecker,
-    EscCharsChecker,
     AssignmentChecker,
     AutoMagicChecker,
     AliasChecker,
@@ -941,10 +738,8 @@ _default_checkers = [
 _default_handlers = [
     PrefilterHandler,
     AliasHandler,
-    ShellEscapeHandler,
     MacroHandler,
     MagicHandler,
     AutoHandler,
-    HelpHandler,
     EmacsHandler
 ]
