@@ -36,7 +36,7 @@ from IPython.core.profiledir import ProfileDir
 from IPython.testing import decorators as dec
 from IPython.testing import tools as tt
 from IPython.utils import py3compat
-
+from IPython.utils.tempdir import TemporaryDirectory
 
 #-----------------------------------------------------------------------------
 # Globals
@@ -151,3 +151,17 @@ def test_list_bundled_profiles():
     bundled_true = [u'cluster', u'math', u'pysh', u'sympy']
     bundled = sorted(list_bundled_profiles())
     nt.assert_equal(bundled, bundled_true)
+
+
+@dec.skipif(sys.version_info < (2,7), "python -m doesn't work on 2.6")
+def test_profile_create_ipython_dir():
+    """ipython profile create respects --ipython-dir"""
+    from subprocess import check_output, STDOUT
+    with TemporaryDirectory() as td:
+        check_output([sys.executable, '-m', 'IPython', 'profile', 'create',
+             'foo', '--ipython-dir=%s' % td], stderr=STDOUT)
+        profile_dir = os.path.join(td, 'profile_foo')
+        assert os.path.exists(profile_dir)
+        ipython_config = os.path.join(profile_dir, 'ipython_config.py')
+        assert os.path.exists(ipython_config)
+        
\ No newline at end of file
