@@ -33,8 +33,9 @@ use of the system.
 import sys
 import threading
 
+from IPython import get_ipython
 from IPython.core.ultratb import AutoFormattedTB
-from IPython.utils.warn import warn, error
+from IPython.utils.warn import error
 
 
 class BackgroundJobManager(object):
@@ -140,6 +141,8 @@ class BackgroundJobManager(object):
         In both cases, the result is stored in the job.result field of the
         background job object.
 
+        You can set `daemon` attribute of the thread by giving the keyword
+        argument `daemon`.
 
         Notes and caveats:
 
@@ -181,7 +184,9 @@ class BackgroundJobManager(object):
             job = BackgroundJobExpr(func_or_exp, glob, loc)
         else:
             raise TypeError('invalid args for new job')
-        
+
+        if kwargs.get('daemon', False):
+            job.daemon = True
         job.num = len(self.all)+1 if self.all else 0
         self.running.append(job)
         self.all[job.num] = job
@@ -371,8 +376,7 @@ class BackgroundJobBase(threading.Thread):
     stat_dead_c = -1
 
     def __init__(self):
-        raise NotImplementedError, \
-              "This class can not be instantiated directly."
+        raise NotImplementedError("This class can not be instantiated directly.")
 
     def _init(self):
         """Common initialization for all BackgroundJob objects"""
