@@ -15,12 +15,10 @@
 import os
 import math
 import random
+import sys
 
 import nose.tools as nt
 
-from nose import with_setup
-
-from IPython.testing import decorators as dec
 from IPython.utils import text
 
 #-----------------------------------------------------------------------------
@@ -32,11 +30,11 @@ def test_columnize():
     size = 5
     items = [l*size for l in 'abc']
     out = text.columnize(items, displaywidth=80)
-    nt.assert_equals(out, 'aaaaa  bbbbb  ccccc\n')
+    nt.assert_equal(out, 'aaaaa  bbbbb  ccccc\n')
     out = text.columnize(items, displaywidth=12)
-    nt.assert_equals(out, 'aaaaa  ccccc\nbbbbb\n')
+    nt.assert_equal(out, 'aaaaa  ccccc\nbbbbb\n')
     out = text.columnize(items, displaywidth=10)
-    nt.assert_equals(out, 'aaaaa\nbbbbb\nccccc\n')
+    nt.assert_equal(out, 'aaaaa\nbbbbb\nccccc\n')
 
 def test_columnize_random():
     """Test with random input to hopfully catch edge case """
@@ -60,31 +58,31 @@ def test_columnize_medium():
     size = 40
     items = [l*size for l in 'abc']
     out = text.columnize(items, displaywidth=80)
-    nt.assert_equals(out, '\n'.join(items+['']))
+    nt.assert_equal(out, '\n'.join(items+['']))
 
 def test_columnize_long():
     """Test columnize with inputs longer than the display window"""
     size = 11
     items = [l*size for l in 'abc']
     out = text.columnize(items, displaywidth=size-1)
-    nt.assert_equals(out, '\n'.join(items+['']))
+    nt.assert_equal(out, '\n'.join(items+['']))
 
 def eval_formatter_check(f):
     ns = dict(n=12, pi=math.pi, stuff='hello there', os=os, u=u"café", b="café")
     s = f.format("{n} {n//4} {stuff.split()[0]}", **ns)
-    nt.assert_equals(s, "12 3 hello")
+    nt.assert_equal(s, "12 3 hello")
     s = f.format(' '.join(['{n//%i}'%i for i in range(1,8)]), **ns)
-    nt.assert_equals(s, "12 6 4 3 2 2 1")
+    nt.assert_equal(s, "12 6 4 3 2 2 1")
     s = f.format('{[n//i for i in range(1,8)]}', **ns)
-    nt.assert_equals(s, "[12, 6, 4, 3, 2, 2, 1]")
+    nt.assert_equal(s, "[12, 6, 4, 3, 2, 2, 1]")
     s = f.format("{stuff!s}", **ns)
-    nt.assert_equals(s, ns['stuff'])
+    nt.assert_equal(s, ns['stuff'])
     s = f.format("{stuff!r}", **ns)
-    nt.assert_equals(s, repr(ns['stuff']))
+    nt.assert_equal(s, repr(ns['stuff']))
     
     # Check with unicode:
     s = f.format("{u}", **ns)
-    nt.assert_equals(s, ns['u'])
+    nt.assert_equal(s, ns['u'])
     # This decodes in a platform dependent manner, but it shouldn't error out
     s = f.format("{b}", **ns)
         
@@ -93,11 +91,11 @@ def eval_formatter_check(f):
 def eval_formatter_slicing_check(f):
     ns = dict(n=12, pi=math.pi, stuff='hello there', os=os)
     s = f.format(" {stuff.split()[:]} ", **ns)
-    nt.assert_equals(s, " ['hello', 'there'] ")
+    nt.assert_equal(s, " ['hello', 'there'] ")
     s = f.format(" {stuff.split()[::-1]} ", **ns)
-    nt.assert_equals(s, " ['there', 'hello'] ")
+    nt.assert_equal(s, " ['there', 'hello'] ")
     s = f.format("{stuff[::2]}", **ns)
-    nt.assert_equals(s, ns['stuff'][::2])
+    nt.assert_equal(s, ns['stuff'][::2])
     
     nt.assert_raises(SyntaxError, f.format, "{n:x}", **ns)
 
@@ -105,12 +103,17 @@ def eval_formatter_no_slicing_check(f):
     ns = dict(n=12, pi=math.pi, stuff='hello there', os=os)
     
     s = f.format('{n:x} {pi**2:+f}', **ns)
-    nt.assert_equals(s, "c +9.869604")
+    nt.assert_equal(s, "c +9.869604")
     
     s = f.format('{stuff[slice(1,4)]}', **ns)
-    nt.assert_equals(s, 'ell')
+    nt.assert_equal(s, 'ell')
     
-    nt.assert_raises(SyntaxError, f.format, "{a[:]}")
+    if sys.version_info >= (3, 4):
+        # String formatting has changed in Python 3.4, so this now works.
+        s = f.format("{a[:]}", a=[1, 2])
+        nt.assert_equal(s, "[1, 2]")
+    else:
+        nt.assert_raises(SyntaxError, f.format, "{a[:]}")
 
 def test_eval_formatter():
     f = text.EvalFormatter()
@@ -129,26 +132,29 @@ def test_dollar_formatter():
     
     ns = dict(n=12, pi=math.pi, stuff='hello there', os=os)
     s = f.format("$n", **ns)
-    nt.assert_equals(s, "12")
+    nt.assert_equal(s, "12")
     s = f.format("$n.real", **ns)
-    nt.assert_equals(s, "12")
+    nt.assert_equal(s, "12")
     s = f.format("$n/{stuff[:5]}", **ns)
-    nt.assert_equals(s, "12/hello")
+    nt.assert_equal(s, "12/hello")
     s = f.format("$n $$HOME", **ns)
-    nt.assert_equals(s, "12 $HOME")
+    nt.assert_equal(s, "12 $HOME")
     s = f.format("${foo}", foo="HOME")
-    nt.assert_equals(s, "$HOME")
+    nt.assert_equal(s, "$HOME")
 
 
 def test_long_substr():
     data = ['hi']
-    nt.assert_equals(text.long_substr(data), 'hi')
+    nt.assert_equal(text.long_substr(data), 'hi')
 
 
 def test_long_substr2():
     data = ['abc', 'abd', 'abf', 'ab']
-    nt.assert_equals(text.long_substr(data), 'ab')
+    nt.assert_equal(text.long_substr(data), 'ab')
 
+def test_long_substr_empty():
+    data = []
+    nt.assert_equal(text.long_substr(data), '')
 
 def test_strip_email():
     src = """\
@@ -161,10 +167,10 @@ def test_strip_email():
 ...   return x+1
 ... 
 >>> zz = f(2.5)"""
-    nt.assert_equals(text.strip_email_quotes(src), cln)
+    nt.assert_equal(text.strip_email_quotes(src), cln)
 
 
 def test_strip_email2():
     src = '> > > list()'
     cln = 'list()'
-    nt.assert_equals(text.strip_email_quotes(src), cln)
+    nt.assert_equal(text.strip_email_quotes(src), cln)
