@@ -14,7 +14,7 @@ if __name__ == '__main__':
     pjoin = os.path.join
     package = 'IPython'
     outdir = pjoin('source','api','generated')
-    docwriter = ApiDocWriter(package,rst_extension='.txt')
+    docwriter = ApiDocWriter(package,rst_extension='.rst')
     # You have to escape the . here because . is a special char for regexps.
     # You must do make clean if you change this!
     docwriter.package_skip_patterns += [r'\.fixes$',
@@ -45,6 +45,7 @@ if __name__ == '__main__':
                                         r'\.ipdoctest',
                                         r'\.Gnuplot',
                                         r'\.frontend\.process\.winprocess',
+                                        r'\.frontend',
                                         r'\.Shell',
                                         ]
     
@@ -57,7 +58,9 @@ if __name__ == '__main__':
                                            r'\.testing\.mkdoctests']
     # Now, generate the outputs
     docwriter.write_api_docs(outdir)
-    docwriter.write_index(outdir, 'gen',
+    # Write index with .txt extension - we can include it, but Sphinx won't try
+    # to compile it
+    docwriter.write_index(outdir, 'gen.txt',
                           relative_to = pjoin('source','api')
                           )
-    print '%d files written' % len(docwriter.written_modules)
+    print ('%d files written' % len(docwriter.written_modules))
