@@ -82,10 +82,7 @@ if __name__ == '__main__':
     try:
         tag = sys.argv[1]
     except IndexError:
-        try:
-            tag = sh2('git describe --exact-match')
-        except CalledProcessError:
-            tag = "dev"   # Fallback
+        tag = "dev"
     
     startdir = os.getcwdu()
     if not os.path.exists(pages_dir):
@@ -117,8 +114,7 @@ if __name__ == '__main__':
 
     try:
         cd(pages_dir)
-        status = sh2('git status | head -1')
-        branch = re.match('\# On branch (.*)$', status).group(1)
+        branch = sh2('git rev-parse --abbrev-ref HEAD').strip()
         if branch != 'gh-pages':
             e = 'On %r, git branch is %r, MUST be "gh-pages"' % (pages_dir,
                                                                  branch)
