@@ -28,6 +28,7 @@ try:
 except:
     from ConfigParser import ConfigParser
 from distutils.command.build_py import build_py
+from distutils.cmd import Command
 from glob import glob
 
 from setupext import install_data_ext
@@ -39,6 +40,7 @@ from setupext import install_data_ext
 # A few handy globals
 isfile = os.path.isfile
 pjoin = os.path.join
+repo_root = os.path.dirname(os.path.abspath(__file__))
 
 def oscmd(s):
     print(">", s)
@@ -71,7 +73,7 @@ def file_doesnt_endwith(test,endings):
 #---------------------------------------------------------------------------
 
 # release.py contains version, authors, license, url, keywords, etc.
-execfile(pjoin('IPython','core','release.py'), globals())
+execfile(pjoin(repo_root, 'IPython','core','release.py'), globals())
 
 # Create a dict with the basic information
 # This dict is eventually passed to setup after additional keys are added.
@@ -132,7 +134,7 @@ def find_package_data():
     
     # walk notebook resources:
     cwd = os.getcwd()
-    os.chdir(os.path.join('IPython', 'frontend', 'html', 'notebook'))
+    os.chdir(os.path.join('IPython', 'html'))
     static_walk = list(os.walk('static'))
     os.chdir(cwd)
     static_data = []
@@ -144,10 +146,15 @@ def find_package_data():
 
     package_data = {
         'IPython.config.profile' : ['README*', '*/*.py'],
+        'IPython.core.tests' : ['*.png', '*.jpg'],
         'IPython.testing' : ['*.txt'],
         'IPython.testing.plugin' : ['*.txt'],
-        'IPython.frontend.html.notebook' : ['templates/*'] + static_data,
-        'IPython.frontend.qt.console' : ['resources/icon/*.svg'],
+        'IPython.html' : ['templates/*'] + static_data,
+        'IPython.qt.console' : ['resources/icon/*.svg'],
+        'IPython.nbconvert' : ['templates/*.tpl', 'templates/latex/*.tplx',
+            'templates/latex/skeleton/*.tplx', 'templates/skeleton/*', 
+            'templates/reveal_internals/*.tpl', 'tests/files/*.*',
+            'exporters/tests/files/*.*']
     }
     return package_data
 
@@ -204,12 +211,12 @@ def find_data_files():
     manpagebase = pjoin('share', 'man', 'man1')
 
     # Simple file lists can be made by hand
-    manpages  = filter(isfile, glob(pjoin('docs','man','*.1.gz')))
+    manpages = [f for f in glob(pjoin('docs','man','*.1.gz')) if isfile(f)]
     if not manpages:
         # When running from a source tree, the manpages aren't gzipped
-        manpages = filter(isfile, glob(pjoin('docs','man','*.1')))
-    igridhelpfiles = filter(isfile,
-                            glob(pjoin('IPython','extensions','igrid_help.*')))
+        manpages = [f for f in glob(pjoin('docs','man','*.1')) if isfile(f)]
+
+    igridhelpfiles = [f for f in glob(pjoin('IPython','extensions','igrid_help.*')) if isfile(f)]
 
     # For nested structures, use the utility above
     example_files = make_dir_struct(
@@ -310,14 +317,14 @@ def find_scripts(entry_points=False, suffix=''):
     """
     if entry_points:
         console_scripts = [s % suffix for s in [
-            'ipython%s = IPython.frontend.terminal.ipapp:launch_new_instance',
+            'ipython%s = IPython:start_ipython',
             'pycolor%s = IPython.utils.PyColorize:main',
             'ipcontroller%s = IPython.parallel.apps.ipcontrollerapp:launch_new_instance',
             'ipengine%s = IPython.parallel.apps.ipengineapp:launch_new_instance',
             'iplogger%s = IPython.parallel.apps.iploggerapp:launch_new_instance',
             'ipcluster%s = IPython.parallel.apps.ipclusterapp:launch_new_instance',
             'iptest%s = IPython.testing.iptest:main',
-            'irunner%s = IPython.lib.irunner:main'
+            'irunner%s = IPython.lib.irunner:main',
         ]]
         gui_scripts = []
         scripts = dict(console_scripts=console_scripts, gui_scripts=gui_scripts)
@@ -349,7 +356,8 @@ def check_for_dependencies():
         print_line, print_raw, print_status,
         check_for_sphinx, check_for_pygments,
         check_for_nose, check_for_pexpect,
-        check_for_pyzmq, check_for_readline
+        check_for_pyzmq, check_for_readline,
+        check_for_jinja2, check_for_tornado
     )
     print_line()
     print_raw("BUILDING IPYTHON")
@@ -366,32 +374,103 @@ def check_for_dependencies():
     check_for_nose()
     check_for_pexpect()
     check_for_pyzmq()
+    check_for_tornado()
     check_for_readline()
+    check_for_jinja2()
 
-def record_commit_info(pkg_dir, build_cmd=build_py):
-    """ Return extended build command class for recording commit
+#---------------------------------------------------------------------------
+# VCS related
+#---------------------------------------------------------------------------
+
+# utils.submodule has checks for submodule status
+execfile(pjoin('IPython','utils','submodule.py'), globals())
+
+class UpdateSubmodules(Command):
+    """Update git submodules
+    
+    IPython's external javascript dependencies live in a separate repo.
+    """
+    description = "Update git submodules"
+    user_options = []
+    
+    def initialize_options(self):
+        pass
+    
+    def finalize_options(self):
+        pass
+    
+    def run(self):
+        failure = False
+        try:
+            self.spawn('git submodule init'.split())
+            self.spawn('git submodule update --recursive'.split())
+        except Exception as e:
+            failure = e
+            print(e)
+        
+        if not check_submodule_status(repo_root) == 'clean':
+            print("submodules could not be checked out")
+            sys.exit(1)
+
+
+def git_prebuild(pkg_dir, build_cmd=build_py):
+    """Return extended build or sdist command class for recording commit
     
     records git commit in IPython.utils._sysinfo.commit
     
     for use in IPython.utils.sysinfo.sys_info() calls after installation.
+    
+    Also ensures that submodules exist prior to running
     """
     
     class MyBuildPy(build_cmd):
         ''' Subclass to write commit data into installation tree '''
         def run(self):
             build_cmd.run(self)
+            # this one will only fire for build commands
+            if hasattr(self, 'build_lib'):
+                self._record_commit(self.build_lib)
+        
+        def make_release_tree(self, base_dir, files):
+            # this one will fire for sdist
+            build_cmd.make_release_tree(self, base_dir, files)
+            self._record_commit(base_dir)
+        
+        def _record_commit(self, base_dir):
             import subprocess
             proc = subprocess.Popen('git rev-parse --short HEAD',
                                     stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE,
                                     shell=True)
             repo_commit, _ = proc.communicate()
-            repo_commit = repo_commit.strip()
-            # We write the installation commit even if it's empty
-            out_pth = pjoin(self.build_lib, pkg_dir, 'utils', '_sysinfo.py')
+            repo_commit = repo_commit.strip().decode("ascii")
+            
+            out_pth = pjoin(base_dir, pkg_dir, 'utils', '_sysinfo.py')
+            if os.path.isfile(out_pth) and not repo_commit:
+                # nothing to write, don't clobber
+                return
+            
+            print("writing git commit '%s' to %s" % (repo_commit, out_pth))
+            
+            # remove to avoid overwriting original via hard link
+            try:
+                os.remove(out_pth)
+            except (IOError, OSError):
+                pass
             with open(out_pth, 'w') as out_file:
                 out_file.writelines([
                     '# GENERATED BY setup.py\n',
-                    'commit = "%s"\n' % repo_commit.decode('ascii'),
+                    'commit = "%s"\n' % repo_commit,
                 ])
-    return MyBuildPy
+    return require_submodules(MyBuildPy)
+
+
+def require_submodules(command):
+    """decorator for instructing a command to check for submodules before running"""
+    class DecoratedCommand(command):
+        def run(self):
+            if not check_submodule_status(repo_root) == 'clean':
+                print("submodules missing! Run `setup.py submodule` and try again")
+                sys.exit(1)
+            command.run(self)
+    return DecoratedCommand
